@@ -2148,7 +2148,8 @@ run_server(void *arg) {
 	CHECK(ns_interfacemgr_create(mctx, sctx, loopmgr, netmgr, dispatchmgr,
 				     NULL, false, &interfacemgr));
 
-	CHECK(dns_view_create(mctx, dns_rdataclass_in, "_default", &view));
+	CHECK(dns_view_create(mctx, dispatchmgr, dns_rdataclass_in, "_default",
+			      &view));
 	CHECK(dns_cache_create(loopmgr, dns_rdataclass_in, "", &cache));
 	dns_view_setcache(view, cache, false);
 	dns_cache_detach(&cache);
@@ -2161,10 +2162,9 @@ run_server(void *arg) {
 	view->qminimization = qmin;
 	view->qmin_strict = qmin_strict;
 
-	CHECK(dns_view_initsecroots(view, mctx));
+	dns_view_initsecroots(view);
 	CHECK(setup_dnsseckeys(NULL, view));
 
-	dns_view_setdispatchmgr(view, dispatchmgr);
 	CHECK(dns_view_createresolver(view, loopmgr, 1, netmgr, 0,
 				      tlsctx_client_cache, dispatch, NULL));
 
