@@ -11,6 +11,8 @@
 # See the COPYRIGHT file distributed with this work for additional
 # information regarding copyright ownership.
 
+set -e
+
 . ../conf.sh
 
 status=0
@@ -30,7 +32,7 @@ do
 	$DIG $DIGOPTS example. @10.53.0.2 soa > dig.out.ns2.test || ret=1
 	if grep SERVFAIL dig.out.ns2.test > /dev/null
 	then
-		try=`expr $try + 1`
+		try=$((try + 1))
 		sleep 1
 	else
                 digcomp dig.out.ns1.test dig.out.ns2.test || ret=1
@@ -39,7 +41,7 @@ do
 done
 echo_i "try $try"
 if [ $ret != 0 ]; then echo_i "failed"; fi
-status=`expr $status + $ret`
+status=$((status + ret))
 
 echo_i "checking that second zone transfer worked"
 ret=0
@@ -49,7 +51,7 @@ do
 	$DIG $DIGOPTS example. @10.53.0.3 soa > dig.out.ns3.test || ret=1
 	if grep SERVFAIL dig.out.ns3.test > /dev/null
 	then
-		try=`expr $try + 1`
+		try=$((try + 1))
 		sleep 1
 	else
                 digcomp dig.out.ns1.test dig.out.ns3.test || ret=1
@@ -58,7 +60,7 @@ do
 done
 echo_i "try $try"
 if [ $ret != 0 ]; then echo_i "failed"; fi
-status=`expr $status + $ret`
+status=$((status + ret))
 
 echo_i "exit status: $status"
 [ $status -eq 0 ] || exit 1
