@@ -20,8 +20,8 @@
 zone=rsasha1.example
 k1=$("$KEYGEN" -a rsasha1 $zone)
 k2=$("$KEYGEN" -a rsasha1 -f KSK $zone)
-cat template.db.in $k1.key $k2.key >$zone.tmp
-# use maximum expiry period (-e 2^31-1-3600)
+cat $zone.db.in $k1.key $k2.key > $zone.tmp
+# use maximum expirey period (-e 2^31-1-3600)
 # use output format full for easy extraction of KSK (-O full)
-"$SIGNER" -e +2147480047 -o $zone -f $zone.db -O full $zone.tmp
+"$SIGNER" +2147480047 -o $zone -f $zone.db -O full $zone.tmp
 rm -f $k1.key $k1.private $k2.key $k2.private $zone.tmp
