@@ -69,8 +69,12 @@ status=$((status + ret))
 # need to complete before reopening/rolling for the counts to
 # be correct.
 
-$DIG $DIGOPTS @10.53.0.3 a.example > dig.out
+echo_i "prime cache"
+ret=0
+$DIG $DIGOPTS @10.53.0.3 a.example > dig.out || true
 wait_for_log 20 "(.): reset client" ns1/named.run || true
+if [ $ret != 0 ]; then echo_i "failed"; fi
+status=$((status + ret))
 
 # check three different dnstap reopen/roll methods:
 # ns1: dnstap-reopen; ns2: dnstap -reopen; ns3: dnstap -roll
@@ -78,17 +82,26 @@ mv ns1/dnstap.out ns1/dnstap.out.save
 mv ns2/dnstap.out ns2/dnstap.out.save
 
 if [ -n "$FSTRM_CAPTURE" ] ; then
+	ret=0
+	echo_i "starting fstrm_capture"
 	$FSTRM_CAPTURE -t protobuf:dnstap.Dnstap -u ns4/dnstap.out \
-		-w dnstap.out > fstrm_capture.out 2>&1 &
+		-w dnstap.out > fstrm_capture.out.1 2>&1 &
 	fstrm_capture_pid=$!
+	wait_for_log 10 "socket path ns4/dnstap.out" fstrm_capture.out.1 || ret=1
+	if [ $ret != 0 ]; then echo_i "failed"; fi
+	status=$((status + ret))
 fi
 
+echo_i "reopen/roll capture streams"
+ret=0
 $RNDCCMD -s 10.53.0.1 dnstap-reopen | sed 's/^/ns1 /' | cat_i
 $RNDCCMD -s 10.53.0.2 dnstap -reopen | sed 's/^/ns2 /' | cat_i
 $RNDCCMD -s 10.53.0.3 dnstap -roll | sed 's/^/ns3 /' | cat_i
 $RNDCCMD -s 10.53.0.4 dnstap -reopen | sed 's/^/ns4 /' | cat_i
 
-$DIG $DIGOPTS @10.53.0.3 a.example > dig.out
+echo_i "send test traffic"
+ret=0
+$DIG $DIGOPTS @10.53.0.3 a.example > dig.out || ret=1
 
 # send an UPDATE to ns2
 $NSUPDATE <<- EOF
@@ -564,7 +577,7 @@ EOF
 
 	echo_i "checking unix socket message counts"
 	sleep 2
-	retry_quiet 5 dnstap_data_ready $fstrm_capture_pid dnstap.out 454 || {
+	retry_quiet 5 dnstap_data_ready $fstrm_capture_pid dnstap.out 450 || {
 		echo_i "dnstap output file smaller than expected"
 		ret=1
 	}
@@ -673,10 +686,14 @@ EOF
 
 	mv dnstap.out dnstap.out.save
 
+	echo_i "restarting fstrm_capture"
 	$FSTRM_CAPTURE -t protobuf:dnstap.Dnstap -u ns4/dnstap.out \
-		-w dnstap.out > fstrm_capture.out 2>&1 &
+		-w dnstap.out > fstrm_capture.out.2 2>&1 &
 	fstrm_capture_pid=$!
-	sleep 1
+	wait_for_log 10 "socket path ns4/dnstap.out" fstrm_capture.out.2 || {
+		echo_i "failed"
+		ret=1
+	}
 	$RNDCCMD -s 10.53.0.4 dnstap -reopen | sed 's/^/ns4 /' | cat_i
 	$DIG $DIGOPTS @10.53.0.4 a.example > dig.out
 
