@@ -11,20 +11,10 @@
 
 import concurrent.futures
 import os
+import subprocess
 import time
-
+import dns.query
 import dns.update
-import pytest
-
-import isctest
-
-pytestmark = pytest.mark.extra_artifacts(
-    [
-        "ns2/zone0*.db",
-        "ns2/zone0*.jnl",
-        "ns*/zone0*.bk",
-    ]
-)
 
 
 def rndc_loop(test_state, server):
@@ -34,7 +24,7 @@ def rndc_loop(test_state, server):
     cmdline = [
         rndc,
         "-c",
-        "../_common/rndc.conf",
+        "../common/rndc.conf",
         "-p",
         port,
         "-s",
@@ -43,11 +33,11 @@ def rndc_loop(test_state, server):
     ]
 
     while not test_state["finished"]:
-        isctest.run.cmd(cmdline, raise_on_exception=False)
+        subprocess.run(cmdline, check=False)
         time.sleep(1)
 
 
-def update_zone(test_state, zone):
+def update_zone(test_state, zone, named_port, logger):
     server = "10.53.0.2"
     for i in range(1000):
         if test_state["finished"]:
@@ -55,18 +45,16 @@ def update_zone(test_state, zone):
         update = dns.update.UpdateMessage(zone)
         update.add(f"dynamic-{i}.{zone}", 300, "TXT", f"txt-{i}")
         try:
-            response = isctest.query.udp(
-                update, server, log_query=False, log_response=False
-            )
+            response = dns.query.udp(update, server, 10, named_port)
             assert response.rcode() == dns.rcode.NOERROR
         except dns.exception.Timeout:
-            isctest.log.info(f"error: query timeout for {zone}")
+            logger.info(f"error: query timeout for {zone}")
 
-    isctest.log.info(f"Update of {server} zone {zone} successful")
+    logger.info(f"Update of {server} zone {zone} successful")
 
 
 # If the test has run to completion without named crashing, it has succeeded.
-def test_update_stress():
+def test_update_stress(named_port, logger):
     test_state = {"finished": False}
 
     with concurrent.futures.ThreadPoolExecutor() as executor:
@@ -75,7 +63,9 @@ def test_update_stress():
         updaters = []
         for i in range(5):
             zone = f"zone00000{i}.example."
-            updaters.append(executor.submit(update_zone, test_state, zone))
+            updaters.append(
+                executor.submit(update_zone, test_state, zone, named_port, logger)
+            )
 
         # All the update_zone() tasks are expected to complete within 5
         # minutes.  If they do not, we cannot assert immediately as that will
