@@ -183,7 +183,7 @@ LLVMFuzzerInitialize(int *argc ISC_ATTR_UNUSED, char ***argv ISC_ATTR_UNUSED) {
 
 	isc_loopmgr_create(mctx, 1, &loopmgr);
 
-	result = dns_view_create(mctx, dns_rdataclass_in, "view", &view);
+	result = dns_view_create(mctx, NULL, dns_rdataclass_in, "view", &view);
 	if (result != ISC_R_SUCCESS) {
 		fprintf(stderr, "dns_view_create failed: %s\n",
 			isc_result_totext(result));
@@ -193,7 +193,7 @@ LLVMFuzzerInitialize(int *argc ISC_ATTR_UNUSED, char ***argv ISC_ATTR_UNUSED) {
 	dns_tsigkeyring_create(mctx, &ring);
 	dns_tsigkeyring_create(mctx, &emptyring);
 
-	result = dns_name_fromstring(name, "tsig-key", 0, NULL);
+	result = dns_name_fromstring(name, "tsig-key", dns_rootname, 0, NULL);
 	if (result != ISC_R_SUCCESS) {
 		fprintf(stderr, "dns_name_fromstring failed: %s\n",
 			isc_result_totext(result));
@@ -214,7 +214,7 @@ LLVMFuzzerInitialize(int *argc ISC_ATTR_UNUSED, char ***argv ISC_ATTR_UNUSED) {
 		return (1);
 	}
 
-	result = dns_name_fromstring(name, "sig0key", 0, NULL);
+	result = dns_name_fromstring(name, "sig0key", dns_rootname, 0, NULL);
 	if (result != ISC_R_SUCCESS) {
 		fprintf(stderr, "dns_name_fromstring failed: %s\n",
 			isc_result_totext(result));
