@@ -3691,25 +3691,15 @@ dns_adb_getquota(dns_adb_t *adb, uint32_t *quotap, uint32_t *freqp,
 		 double *lowp, double *highp, double *discountp) {
 	REQUIRE(DNS_ADB_VALID(adb));
 
-	if (quotap != NULL) {
-		*quotap = adb->quota;
-	}
+	SET_IF_NOT_NULL(quotap, adb->quota);
 
-	if (freqp != NULL) {
-		*freqp = adb->atr_freq;
-	}
+	SET_IF_NOT_NULL(freqp, adb->atr_freq);
 
-	if (lowp != NULL) {
-		*lowp = adb->atr_low;
-	}
+	SET_IF_NOT_NULL(lowp, adb->atr_low);
 
-	if (highp != NULL) {
-		*highp = adb->atr_high;
-	}
+	SET_IF_NOT_NULL(highp, adb->atr_high);
 
-	if (discountp != NULL) {
-		*discountp = adb->atr_discount;
-	}
+	SET_IF_NOT_NULL(discountp, adb->atr_discount);
 }
 
 static bool
