@@ -116,8 +116,10 @@ typedef struct dns_message	   dns_message_t;
 typedef uint16_t		   dns_messageid_t;
 typedef isc_region_t		   dns_label_t;
 typedef struct dns_name		   dns_name_t;
+typedef struct dns_nametree	   dns_nametree_t;
 typedef ISC_LIST(dns_name_t) dns_namelist_t;
 typedef struct dns_ntatable	 dns_ntatable_t;
+typedef struct dns_ntnode	 dns_ntnode_t;
 typedef uint16_t		 dns_opcode_t;
 typedef struct dns_order	 dns_order_t;
 typedef struct dns_peer		 dns_peer_t;
@@ -434,7 +436,7 @@ typedef isc_result_t (*dns_additionaldatafunc_t)(
 
 typedef isc_result_t (*dns_digestfunc_t)(void *, isc_region_t *);
 
-typedef void (*dns_xfrindone_t)(dns_zone_t *, isc_result_t);
+typedef void (*dns_xfrindone_t)(dns_zone_t *, uint32_t *, isc_result_t);
 
 typedef void (*dns_updatecallback_t)(void *, isc_result_t, dns_message_t *);
 
