@@ -140,12 +140,11 @@ struct dns_view {
 	dns_acl_t	     *denyansweracl;
 	dns_acl_t	     *nocasecompress;
 	bool		      msgcompression;
-	dns_rbt_t	     *answeracl_exclude;
-	dns_rbt_t	     *denyanswernames;
-	dns_rbt_t	     *answernames_exclude;
+	dns_nametree_t	     *answeracl_exclude;
+	dns_nametree_t	     *denyanswernames;
+	dns_nametree_t	     *answernames_exclude;
+	dns_nametree_t	     *sfd;
 	dns_rrl_t	     *rrl;
-	dns_rbt_t	     *sfd;
-	isc_rwlock_t	      sfd_lock;
 	bool		      provideixfr;
 	bool		      requestnsid;
 	bool		      sendcookie;
@@ -259,8 +258,8 @@ struct dns_view {
 #endif /* HAVE_LMDB */
 
 isc_result_t
-dns_view_create(isc_mem_t *mctx, dns_rdataclass_t rdclass, const char *name,
-		dns_view_t **viewp);
+dns_view_create(isc_mem_t *mctx, dns_dispatchmgr_t *dispmgr,
+		dns_rdataclass_t rdclass, const char *name, dns_view_t **viewp);
 /*%<
  * Create a view.
  *
@@ -378,9 +377,6 @@ dns_view_createresolver(dns_view_t *view, isc_loopmgr_t *loopmgr,
  *
  *\li	'view' does not have a resolver already.
  *
- *\li	A dispatch manager has been associated with the view by calling
- *	dns_view_setdispatchmgr().
- *
  *\li	The requirements of dns_resolver_create() apply to 'ndisp',
  *	'netmgr', 'options', 'tlsctx_cache', 'dispatchv4', and 'dispatchv6'.
  *
@@ -932,17 +928,14 @@ dns_view_iscacheshared(dns_view_t *view);
  *\li	#false otherwise.
  */
 
-isc_result_t
+void
 dns_view_initntatable(dns_view_t *view, isc_loopmgr_t *loopmgr);
 /*%<
  * Initialize the negative trust anchor table for the view.
  *
  * Requires:
  * \li	'view' is valid.
- *
- * Returns:
- *\li	ISC_R_SUCCESS
- *\li	Any other result indicates failure
+ * \li	'loopmgr' is a valid loopmgr.
  */
 
 isc_result_t
@@ -963,8 +956,8 @@ dns_view_getntatable(dns_view_t *view, dns_ntatable_t **ntp);
  *\li	ISC_R_NOTFOUND
  */
 
-isc_result_t
-dns_view_initsecroots(dns_view_t *view, isc_mem_t *mctx);
+void
+dns_view_initsecroots(dns_view_t *view);
 /*%<
  * Initialize security roots for the view, detaching any previously
  * existing security roots first.  (Note that secroots_priv is
@@ -1257,12 +1250,10 @@ dns_view_getudpsize(dns_view_t *view);
  * Get the current EDNS UDP buffer size.
  */
 
-void
-dns_view_setdispatchmgr(dns_view_t *view, dns_dispatchmgr_t *dispatchmgr);
 dns_dispatchmgr_t *
 dns_view_getdispatchmgr(dns_view_t *view);
 /*%<
- * Set/get the dispatch manager for the view; this wil be used
+ * Get the attached dispatch manager for the view; this will be used
  * by the resolver and request managers to send and receive DNS
  * messages.
  */
