@@ -224,7 +224,7 @@ dns_rdataslab_fromrdataset(dns_rdataset_t *rdataset, isc_mem_t *mctx,
 	 * Remember the original number of items.
 	 */
 	nalloc = nitems;
-	x = isc_mem_get(mctx, nalloc * sizeof(struct xrdata));
+	x = isc_mem_cget(mctx, nalloc, sizeof(struct xrdata));
 
 	/*
 	 * Save all of the rdata members into an array.
@@ -329,12 +329,11 @@ dns_rdataslab_fromrdataset(dns_rdataset_t *rdataset, isc_mem_t *mctx,
 	 * Allocate the memory, set up a buffer, start copying in
 	 * data.
 	 */
-	rawbuf = isc_mem_getx(mctx, buflen, ISC_MEM_ZERO);
+	rawbuf = isc_mem_cget(mctx, 1, buflen);
 
 #if DNS_RDATASET_FIXED
 	/* Allocate temporary offset table. */
-	offsettable = isc_mem_getx(mctx, nalloc * sizeof(unsigned int),
-				   ISC_MEM_ZERO);
+	offsettable = isc_mem_cget(mctx, nalloc, sizeof(unsigned int));
 #endif /* if DNS_RDATASET_FIXED */
 
 	region->base = rawbuf;
@@ -385,13 +384,13 @@ dns_rdataslab_fromrdataset(dns_rdataset_t *rdataset, isc_mem_t *mctx,
 
 #if DNS_RDATASET_FIXED
 	fillin_offsets(offsetbase, offsettable, nalloc);
-	isc_mem_put(mctx, offsettable, nalloc * sizeof(unsigned int));
+	isc_mem_cput(mctx, offsettable, nalloc, sizeof(unsigned int));
 #endif /* if DNS_RDATASET_FIXED */
 
 	result = ISC_R_SUCCESS;
 
 free_rdatas:
-	isc_mem_put(mctx, x, nalloc * sizeof(struct xrdata));
+	isc_mem_cput(mctx, x, nalloc, sizeof(struct xrdata));
 	return (result);
 }
 
@@ -685,8 +684,8 @@ dns_rdataslab_merge(unsigned char *oslab, unsigned char *nslab,
 	 */
 	tcurrent += (tcount * 4);
 
-	offsettable = isc_mem_getx(
-		mctx, (ocount + oncount) * sizeof(unsigned int), ISC_MEM_ZERO);
+	offsettable = isc_mem_cget(mctx, (ocount + oncount),
+				   sizeof(unsigned int));
 #endif /* if DNS_RDATASET_FIXED */
 
 	/*
@@ -791,8 +790,8 @@ dns_rdataslab_merge(unsigned char *oslab, unsigned char *nslab,
 #if DNS_RDATASET_FIXED
 	fillin_offsets(offsetbase, offsettable, ocount + oncount);
 
-	isc_mem_put(mctx, offsettable,
-		    (ocount + oncount) * sizeof(unsigned int));
+	isc_mem_cput(mctx, offsettable, (ocount + oncount),
+		     sizeof(unsigned int));
 #endif /* if DNS_RDATASET_FIXED */
 
 	INSIST(tcurrent == tstart + tlength);
@@ -909,8 +908,7 @@ dns_rdataslab_subtract(unsigned char *mslab, unsigned char *sslab,
 #if DNS_RDATASET_FIXED
 	offsetbase = tcurrent;
 
-	offsettable = isc_mem_getx(mctx, mcount * sizeof(unsigned int),
-				   ISC_MEM_ZERO);
+	offsettable = isc_mem_cget(mctx, mcount, sizeof(unsigned int));
 #endif /* if DNS_RDATASET_FIXED */
 
 	/*
@@ -966,7 +964,7 @@ dns_rdataslab_subtract(unsigned char *mslab, unsigned char *sslab,
 #if DNS_RDATASET_FIXED
 	fillin_offsets(offsetbase, offsettable, mcount);
 
-	isc_mem_put(mctx, offsettable, mcount * sizeof(unsigned int));
+	isc_mem_cput(mctx, offsettable, mcount, sizeof(unsigned int));
 #endif /* if DNS_RDATASET_FIXED */
 
 	INSIST(tcurrent == tstart + tlength);
