@@ -216,8 +216,8 @@ isc_netmgr_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr, isc_nm_t **netmgrp) {
 	atomic_init(&netmgr->keepalive, 30000);
 	atomic_init(&netmgr->advertised, 30000);
 
-	netmgr->workers =
-		isc_mem_get(mctx, netmgr->nloops * sizeof(netmgr->workers[0]));
+	netmgr->workers = isc_mem_cget(mctx, netmgr->nloops,
+				       sizeof(netmgr->workers[0]));
 
 	isc_loopmgr_teardown(loopmgr, netmgr_teardown, netmgr);
 
@@ -239,6 +239,7 @@ isc_netmgr_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr, isc_nm_t **netmgrp) {
 
 		isc_mempool_create(worker->mctx, sizeof(isc__nm_uvreq_t),
 				   &worker->uvreq_pool);
+		isc_mempool_setfreemax(worker->uvreq_pool, ISC_NM_UVREQS_MAX);
 
 		isc_loop_attach(loop, &worker->loop);
 		isc_loop_teardown(loop, networker_teardown, worker);
@@ -268,8 +269,8 @@ nm_destroy(isc_nm_t **mgr0) {
 
 	isc_mutex_destroy(&mgr->lock);
 
-	isc_mem_put(mgr->mctx, mgr->workers,
-		    mgr->nloops * sizeof(mgr->workers[0]));
+	isc_mem_cput(mgr->mctx, mgr->workers, mgr->nloops,
+		     sizeof(mgr->workers[0]));
 	isc_mem_putanddetach(&mgr->mctx, mgr, sizeof(*mgr));
 }
 
@@ -384,21 +385,13 @@ isc_nm_gettimeouts(isc_nm_t *mgr, uint32_t *initial, uint32_t *idle,
 		   uint32_t *keepalive, uint32_t *advertised) {
 	REQUIRE(VALID_NM(mgr));
 
-	if (initial != NULL) {
-		*initial = atomic_load_relaxed(&mgr->init);
-	}
+	SET_IF_NOT_NULL(initial, atomic_load_relaxed(&mgr->init));
 
-	if (idle != NULL) {
-		*idle = atomic_load_relaxed(&mgr->idle);
-	}
+	SET_IF_NOT_NULL(idle, atomic_load_relaxed(&mgr->idle));
 
-	if (keepalive != NULL) {
-		*keepalive = atomic_load_relaxed(&mgr->keepalive);
-	}
+	SET_IF_NOT_NULL(keepalive, atomic_load_relaxed(&mgr->keepalive));
 
-	if (advertised != NULL) {
-		*advertised = atomic_load_relaxed(&mgr->advertised);
-	}
+	SET_IF_NOT_NULL(advertised, atomic_load_relaxed(&mgr->advertised));
 }
 
 bool
@@ -464,8 +457,8 @@ nmsocket_cleanup(void *arg) {
 		/*
 		 * Now free them.
 		 */
-		isc_mem_put(sock->worker->mctx, sock->children,
-			    sock->nchildren * sizeof(*sock));
+		isc_mem_cput(sock->worker->mctx, sock->children,
+			     sock->nchildren, sizeof(*sock));
 		sock->children = NULL;
 		sock->nchildren = 0;
 	}
@@ -814,6 +807,8 @@ dequeue_handle(isc_nmsocket_t *sock) {
 	if (handle != NULL) {
 		ISC_LIST_DEQUEUE(sock->inactive_handles, handle, inactive_link);
 
+		sock->inactive_handles_cur--;
+
 		isc_refcount_init(&handle->references, 1);
 		INSIST(VALID_NMHANDLE(handle));
 		return (handle);
@@ -921,7 +916,10 @@ nmhandle__destroy(isc_nmhandle_t *handle) {
 #if defined(__SANITIZE_ADDRESS__) || defined(__SANITIZE_THREAD__)
 	nmhandle_free(sock, handle);
 #else
-	if (sock->active) {
+	if (sock->active &&
+	    sock->inactive_handles_cur < sock->inactive_handles_max)
+	{
+		sock->inactive_handles_cur++;
 		ISC_LIST_APPEND(sock->inactive_handles, handle, inactive_link);
 	} else {
 		nmhandle_free(sock, handle);
@@ -2225,8 +2223,8 @@ set_tlsctx_workers(isc_nmsocket_t *listener, isc_tlsctx_t *tlsctx) {
 	for (size_t i = 0; i < nworkers; i++) {
 		isc__networker_t *worker =
 			&listener->worker->netmgr->workers[i];
-		settlsctx_data_t *data = isc_mem_getx(
-			worker->loop->mctx, sizeof(*data), ISC_MEM_ZERO);
+		settlsctx_data_t *data = isc_mem_cget(worker->loop->mctx, 1,
+						      sizeof(*data));
 
 		isc__nmsocket_attach(listener, &data->listener);
 		isc_tlsctx_attach(tlsctx, &data->tlsctx);
