@@ -195,8 +195,7 @@ isc__tls_initialize(void) {
 	RUNTIME_CHECK(OPENSSL_init_ssl(opts, NULL) == 1);
 #else
 	nlocks = CRYPTO_num_locks();
-	locks = isc_mem_getx(isc__tls_mctx, nlocks * sizeof(locks[0]),
-			     ISC_MEM_ZERO);
+	locks = isc_mem_cget(isc__tls_mctx, nlocks, sizeof(locks[0]));
 	isc_mutexblock_init(locks, nlocks);
 	CRYPTO_set_locking_callback(isc__tls_lock_callback);
 	CRYPTO_THREADID_set_callback(isc__tls_set_thread_id);
@@ -245,7 +244,7 @@ isc__tls_shutdown(void) {
 
 	if (locks != NULL) {
 		isc_mutexblock_destroy(locks, nlocks);
-		isc_mem_put(isc__tls_mctx, locks, nlocks * sizeof(locks[0]));
+		isc_mem_cput(isc__tls_mctx, locks, nlocks, sizeof(locks[0]));
 		locks = NULL;
 	}
 #endif
@@ -845,10 +844,12 @@ isc_tls_create(isc_tlsctx_t *ctx) {
 
 void
 isc_tls_free(isc_tls_t **tlsp) {
+	isc_tls_t *tls = NULL;
 	REQUIRE(tlsp != NULL && *tlsp != NULL);
 
-	SSL_free(*tlsp);
+	tls = *tlsp;
 	*tlsp = NULL;
+	SSL_free(tls);
 }
 
 const char *
@@ -1044,6 +1045,7 @@ isc_tlsctx_enable_peer_verification(isc_tlsctx_t *tlsctx, const bool is_server,
 			ret = X509_VERIFY_PARAM_set1_host(param, hostname, 0);
 		}
 		if (ret != 1) {
+			ERR_clear_error();
 			return (ISC_R_FAILURE);
 		}
 
@@ -1094,6 +1096,7 @@ isc_tlsctx_load_client_ca_names(isc_tlsctx_t *ctx, const char *ca_bundle_file) {
 
 	cert_names = SSL_load_client_CA_file(ca_bundle_file);
 	if (cert_names == NULL) {
+		ERR_clear_error();
 		return (ISC_R_FAILURE);
 	}
 
@@ -1134,6 +1137,7 @@ isc_tls_cert_store_create(const char *ca_bundle_filename,
 	return (ISC_R_SUCCESS);
 
 error:
+	ERR_clear_error();
 	if (store != NULL) {
 		X509_STORE_free(store);
 	}
@@ -1615,6 +1619,7 @@ isc_tlsctx_client_session_cache_keep(isc_tlsctx_client_session_cache_t *cache,
 
 	sess = SSL_get1_session(tls);
 	if (sess == NULL) {
+		ERR_clear_error();
 		return;
 	} else if (!ssl_session_seems_resumable(sess)) {
 		SSL_SESSION_free(sess);
