@@ -115,7 +115,7 @@ cleanup:
 }
 
 void
-shutdown_interfacemgr(void *arg __attribute__((__unused__))) {
+shutdown_interfacemgr(void *arg ISC_ATTR_UNUSED) {
 	if (interfacemgr != NULL) {
 		ns_interfacemgr_shutdown(interfacemgr);
 		ns_interfacemgr_detach(&interfacemgr);
@@ -285,7 +285,7 @@ attach_query_msg_to_client(ns_client_t *client, const char *qnamestr,
 	 * class IN and type "qtype", link the two and add the result to the
 	 * QUESTION section of the query.
 	 */
-	result = dns_name_fromstring(qname, qnamestr, 0, mctx);
+	result = dns_name_fromstring(qname, qnamestr, dns_rootname, 0, mctx);
 	if (result != ISC_R_SUCCESS) {
 		goto put_name;
 	}
@@ -434,12 +434,13 @@ ns_test_qctx_create(const ns_test_qctx_create_params_t *params,
 	if (result != ISC_R_SUCCESS) {
 		return (result);
 	}
-	TIME_NOW(&client->tnow);
+	client->tnow = isc_time_now();
 
 	/*
 	 * Every client needs to belong to a view.
 	 */
-	result = dns_test_makeview("view", params->with_cache, &client->view);
+	result = dns_test_makeview("view", false, params->with_cache,
+				   &client->view);
 	if (result != ISC_R_SUCCESS) {
 		goto detach_client;
 	}
@@ -456,7 +457,7 @@ ns_test_qctx_create(const ns_test_qctx_create_params_t *params,
 
 	/*
 	 * Allow recursion for the client.  As NS_CLIENTATTR_RA normally gets
-	 * set in ns__client_request(), i.e. earlier than the unit tests hook
+	 * set in ns_client_request(), i.e. earlier than the unit tests hook
 	 * into the call chain, just set it manually.
 	 */
 	client->attributes |= NS_CLIENTATTR_RA;
@@ -532,7 +533,7 @@ ns_test_loaddb(dns_db_t **db, dns_dbtype_t dbtype, const char *origin,
 
 	name = dns_fixedname_initname(&fixed);
 
-	result = dns_name_fromstring(name, origin, 0, NULL);
+	result = dns_name_fromstring(name, origin, dns_rootname, 0, NULL);
 	if (result != ISC_R_SUCCESS) {
 		return (result);
 	}
